Documentation
¶
Index ¶
Constants ¶
const (
ALPHA = float32(1.0)
)
Variables ¶
var (
Verbose = false
)
Functions ¶
This section is empty.
Types ¶
type Link ¶
type Link struct {
// Source node of the link
FromNode string
// Target node of the link
ToNode string
// Corresponding route
RouteID string
// In most cases this should trave time along the link
TravelCost float32
// Interval of public transport
// Headway could have only dwell (on-board) links
Headway float32
}
Link is just an edge in graph
type PriorityQueue ¶
type PriorityQueue []*pqEntry
func (PriorityQueue) Len ¶
func (pq PriorityQueue) Len() int
func (PriorityQueue) Less ¶
func (pq PriorityQueue) Less(i, j int) bool
func (*PriorityQueue) Pop ¶
func (pq *PriorityQueue) Pop() interface{}
func (*PriorityQueue) Push ¶
func (pq *PriorityQueue) Push(x interface{})
func (PriorityQueue) Swap ¶
func (pq PriorityQueue) Swap(i, j int)
type Strategy ¶
type Strategy struct {
// u_{i} - expected time to destination
Labels map[string]float32
// f_{i} - combined frequency at node
Freqs map[string]float32
// \overline{A} - attractive links in hyperpath
ASet []*Link
}
Strategy is optimal strategy as it defined in Spiess-Florian algorithm
Source Files
¶
Click to show internal directories.
Click to hide internal directories.